#! /usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Dependencies
 */
var child_process_1 = require("child_process");
var fs_1 = require("fs");
var moment_1 = __importDefault(require("moment"));
var prompts_1 = __importDefault(require("prompts"));
var signale_1 = __importDefault(require("signale"));
var util_1 = require("util");
var execAsync = util_1.promisify(child_process_1.exec);
/**
 * Information sources
 */
// tslint:disable-next-line: no-var-requires
var version = require(process.cwd() + "/package.json").version;
var args = process.argv.slice(2);
var opts = [
    "--help",
    "--init",
    "--no-hash",
    "--no-user",
    "--no-version",
    "--no-time",
    "--no-message",
];
/**
 * Get relevant build information
 */
function getGitUser() {
    return __awaiter(this, void 0, void 0, function () {
        var _a, stdout, stderr;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, execAsync("git config user.name")];
                case 1:
                    _a = _b.sent(), stdout = _a.stdout, stderr = _a.stderr;
                    if (stderr) {
                        signale_1.default.error("Error getting git user, skipping...");
                        return [2 /*return*/, undefined];
                    }
                    else {
                        signale_1.default.success("Found git user:", stdout.replace("\n", ""));
                        return [2 /*return*/, stdout.replace("\n", "")];
                    }
                    return [2 /*return*/];
            }
        });
    });
}
function getCommitHash() {
    return __awaiter(this, void 0, void 0, function () {
        var _a, stdout, stderr;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, execAsync("git rev-parse --short HEAD")];
                case 1:
                    _a = _b.sent(), stdout = _a.stdout, stderr = _a.stderr;
                    if (stderr) {
                        signale_1.default.error("Error getting latest commit hash, skipping...");
                        return [2 /*return*/, undefined];
                    }
                    else {
                        signale_1.default.success("Found commit hash:", stdout.replace("\n", ""));
                        return [2 /*return*/, stdout.replace("\n", "")];
                    }
                    return [2 /*return*/];
            }
        });
    });
}
function getBuildMessage() {
    return __awaiter(this, void 0, void 0, function () {
        var response;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, prompts_1.default({
                        type: "text",
                        name: "message",
                        message: "Add build message? (optional)",
                    })];
                case 1:
                    response = _a.sent();
                    return [2 /*return*/, response.message === "" ? undefined : response.message];
            }
        });
    });
}
function buildInfo() {
    return __awaiter(this, void 0, void 0, function () {
        var build, _a, _b, _c, exportPath, _d;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    signale_1.default.start("Collecting build information...");
                    // Check provided args if they exist
                    args.map(function (arg) {
                        if (!opts.includes(arg)) {
                            signale_1.default.error("Error: Unknown arg \"" + arg + "\", please re-check arguments before running the tool again!");
                            process.exit(1);
                        }
                    });
                    build = {};
                    if (!!args.includes("--no-hash")) return [3 /*break*/, 2];
                    _a = build;
                    return [4 /*yield*/, getCommitHash()];
                case 1:
                    _a.hash = _e.sent();
                    _e.label = 2;
                case 2:
                    if (!!args.includes("--no-user")) return [3 /*break*/, 4];
                    _b = build;
                    return [4 /*yield*/, getGitUser()];
                case 3:
                    _b.user = _e.sent();
                    _e.label = 4;
                case 4:
                    if (!!args.includes("--no-message")) return [3 /*break*/, 6];
                    _c = build;
                    return [4 /*yield*/, getBuildMessage()];
                case 5:
                    _c.message = _e.sent();
                    _e.label = 6;
                case 6:
                    if (!args.includes("--no-version")) {
                        build.version = version;
                    }
                    if (!args.includes("--no-time")) {
                        build.timestamp = moment_1.default().format("MMMM DD, YYYY HH:mm:ss");
                    }
                    signale_1.default.info(build);
                    exportPath = process.cwd() + "/src/build.ts";
                    _e.label = 7;
                case 7:
                    _e.trys.push([7, 9, , 10]);
                    return [4 /*yield*/, fs_1.promises.writeFile(exportPath, "// Angular build information, automatically generated by `angular-build-info`\n" +
                            "export const buildInfo = " +
                            JSON.stringify(build, null, 4).replace(/\"([^(\")"]+)\":/g, "$1:") +
                            ";\n")];
                case 8:
                    _e.sent();
                    signale_1.default.success("Saved build information to `" + exportPath + "`");
                    return [3 /*break*/, 10];
                case 9:
                    _d = _e.sent();
                    signale_1.default.error("An error occured writing to `" + exportPath + "`, does the path exist?");
                    return [3 /*break*/, 10];
                case 10: return [2 /*return*/];
            }
        });
    });
}
function init() {
    return __awaiter(this, void 0, void 0, function () {
        var exportPath, template, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    exportPath = process.cwd() + "/src/build.ts";
                    signale_1.default.info("Welcome to `angular-build-info`!");
                    signale_1.default.info("We will now create a boilerplate `build.ts` file in `" + exportPath + "` and fill it with basic information so you can start implementing it in your front-end.");
                    signale_1.default.info("If you wish for more info on how to implement the provided info in your Angular app, feel free to check out the main repo over at https://github.com/4dams/angular-build-info");
                    signale_1.default.start("Creating `build.ts` file...");
                    template = {
                        hash: "1e872b5",
                        timestamp: moment_1.default().format("MMMM DD, YYYY HH:mm:ss"),
                        user: "Octocat",
                        version: "1.0.0",
                    };
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, fs_1.promises.writeFile(exportPath, "// Angular build information, automatically generated by `angular-build-info`\n" +
                            "export const buildInfo = " +
                            JSON.stringify(template, null, 4).replace(/\"([^(\")"]+)\":/g, "$1:") +
                            ";\n")];
                case 2:
                    _b.sent();
                    signale_1.default.success("Successfully created `build.ts` file!");
                    signale_1.default.info("You should now modify your build/deploy scripts in your `package.json` file to run this script every time before your Angular app is built. An example would be:");
                    signale_1.default.info("[...] \"build\": \"build-info && ng build --prod\", [...]");
                    signale_1.default.info("Again, you can find more info on implementing this tool on the main repo.");
                    return [3 /*break*/, 4];
                case 3:
                    _a = _b.sent();
                    signale_1.default.error("An error occured writing to `" + exportPath + "`, does the path exist?");
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    });
}
function displayManual() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            signale_1.default.info("Welcome to `angular-build-info`!");
            signale_1.default.info("");
            signale_1.default.info("--help          Displays this message");
            signale_1.default.info("--init          Creates template `build.ts` file so you can start implementing it");
            signale_1.default.info("--no-hash       Will not add latest commit hash to final `build.ts`");
            signale_1.default.info("--no-user       Will not add git username to final `build.ts`");
            signale_1.default.info("--no-version    Will not add version from `package.json` to `build.ts`");
            signale_1.default.info("--no-time       Will not add timestamp to final `build.ts`");
            return [2 /*return*/];
        });
    });
}
/**
 * Main function we run upon starting the script
 */
function start() {
    if (args.includes("--init")) {
        init();
        return;
    }
    if (args.includes("--help")) {
        displayManual();
        return;
    }
    buildInfo();
}
start();
